/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_ASSETS_MANAGER_HPP
#define FGGL_ASSETS_MANAGER_HPP

#include <string_view>
#include <map>
#include <functional>
#include <memory>

#include "fggl/data/storage.hpp"
#include "fggl/util/safety.hpp"

namespace fggl::assets {

	struct AssetTag{};
	using AssetType = util::OpaqueName<std::string_view, AssetTag>;

	struct AssetData {
		void* data;
		std::size_t size;
	};

	struct AssetCallbacks {
		std::function<void(const AssetType&, AssetData)> init;
		std::function<void(const AssetType&, AssetData)> destroy;
	};

	struct Asset{};

	class AssetManager {
		public:
			constexpr const static modules::ModuleService service = modules::make_service("fggl::assets::AssetModule");
			using AssetGUID = std::string;

			AssetManager(data::Storage* storage) : m_storage(storage) {}
			virtual ~AssetManager() = default;

			void load(const AssetType&, const AssetGUID& name);
			void loadToTemp(const AssetType&, const AssetGUID& name);
			void unload(const AssetGUID& name);

		private:
			data::Storage* m_storage;
			std::map<AssetGUID, std::shared_ptr<Asset>> m_registry;
			std::map<AssetType, AssetCallbacks> m_callbacks;
	};


} // namespace fggl::assets

#endif //FGGL_ASSETS_MANAGER_HPP
